# !/bin/bash
#
# ax4g_install.sh - Install Script for AX4G card 
#
# AX4G card add the feature of sending GSM messages
# Copyright (C) 20010-2012 Atcom Communication Co. Ltd,
# 

PROD_HOME=`pwd`
DRIVER_VERSION='1.0.1'
SYSTEM_KERN=`uname -r`
DAHDI_MODULES_DIR="/lib/modules/$SYSTEM_KERN/dahdi"
# patch, src patch
PROD_PATCH_DIR=$PROD_HOME/patches
DRIVER_SRC_DIR=$PROD_HOME/src
LOG_DIR=$PROD_HOME/log
DAHDI_TOOL_CONFIG_NEED_BACKUP="NO" 
# log file
LOG_FILE="$LOG_DIR/log.$(date +%F-%k-%M)"

EXECUTE_UID=0
LOG_WFLAG=YES

# asterisk and dahdi src code path
AST_DEFAULT_INSTALL_DIR="/usr/src/asterisk"
DAHDI_LINUX_DEFAULT_INSTALL_DIR="/usr/src/dahdi-linux"
DAHDI_TOOLS_DEFAULT_INSTALL_DIR="/usr/src/dahdi-tools"
DAHDI_LINUX_COMPLETE_DEFAULT_INSTALL_DIR="/usr/src/dahdi-linux-complete"
DAHDI_LINUX_SOURCE_DIR=$DAHDI_LINUX_DEFAULT_INSTALL_DIR
DAHDI_TOOLS_SOURCE_DIR=$DAHDI_TOOLS_DEFAULT_INSTALL_DIR
DAHDI_LINUX_COMPLETE_SOURCE_DIR=$DAHDI_LINUX_DEFAULT_COMPLETE_INSTALL_DIR
AST_SOURCE_DIR=$AST_DEFAULT_INSTALL_DIR

PROD_PATCH_DIR=$PROD_HOME/patches
AST_MODULES="/usr/lib/asterisk/modules"
AST_CONFIG_DIR="/etc/asterisk"
AST_INCLUDE_DIR="/usr/include/asterisk"
AST_SBIN_DIR="/usr/sbin"
DAHDI_CFG_DIR="/etc/dahdi"
DAHDI_MODULES_CFG="/etc/dahdi/modules"

DHAID_AUTO_INSTALL="YES"
AST_AUTO_INSTALL="YES"

# system , elastix  VERSION
NOW_ELASTIX_VERSION=""
SYSTEM_VERSION=""

SUPPORT_BARE_INSTALL=$1

# ----------------------------------------------------------------------------
# brush the term  screen funcation named brush() clearscr()
# ----------------------------------------------------------------------------
brush()
{
	if test $TESTACTIVE; then
		return
	fi

	# clear term screen 
	[ "$TERM" ] && clear 2> /dev/null
}


# ----------------------------------------------------------------------------
# check bash
# ----------------------------------------------------------------------------
check_sh()
{
	BASH_ENABLE=`echo $BASH_VERSION | cut -d'.' -f1 2> /dev/null`
	if test -z $BASH_ENABLE; then
		echo "Bash tool is not find in the computer!"
		exit 1
	fi
}


# ----------------------------------------------------------------------------
# the title of running install script
# ----------------------------------------------------------------------------
display()
{
	if test -z $TESTACTIVE; then
		brush
	fi
	
	echo -e "**************************************************************************"
	echo -e "@@                   ATCOM AX4G Card Install Script                     @@"
	echo -e "@@                             V1.0.1                                   @@"
	echo -e "@@                         Welcome to Use AX4G                          @@"
	echo -e "@@                 Copyright(c) 2010-2012 ATCOM  Co., Ltd.              @@"
	echo -e "**************************************************************************"	
	echo ""
	echo ""

	return 0
}


# ----------------------------------------------------------------------------
# write debug informations into log file
# ----------------------------------------------------------------------------
debug_log()
{
	if [ "$2" == "0" ]; then
		:
	else
		echo -ne "$1"
	fi
	if [ "$LOG_WFLAG" == "YES" ]; then
		if [ "$3" == "0" ]; then
			:
		else
			echo -ne "$(LANG=C date) : Debug: $1" >> "$LOG_FILE"
		fi
	fi
}


# ----------------------------------------------------------------------------
#  write error  informations into log file
# ----------------------------------------------------------------------------
error_log()
{
	echo -ne "Error: $*" >&2

	if [ "$LOG_WFLAG" == "YES" ]; then
		echo -ne "$(LANG=C date) : Error: $*" >> "$LOG_FILE"
	fi
}


# ----------------------------------------------------------------------------
# wait for some seconds
# ----------------------------------------------------------------------------
wait()
{
	[ $# -ne 0 ] && sleep $1 >&2 >> /dev/null && return 0
	echo -e "Press [Enter] to continue install!\c"
	read result
	
	return 0
}

# ----------------------------------------------------------------------------
# Prompt user to  input 
# ----------------------------------------------------------------------------
prompt()
{
	if test $TESTACTIVE; then
		return 0
	fi

	echo -ne "$*" >&2
	read INPUT rest
	return 0
}

# ----------------------------------------------------------------------------
# Get input result Yes or No
# ----------------------------------------------------------------------------
get_res()
{
	if test $TESTACTIVE; then
		return 0
	fi

	while prompt "$* (y/n) "
	do	case $INPUT in
			[yY])	return 0
				;;
			[nN])	return 1
				;;
			*)	echo -e "\nPlease input y(Y) or n(N)" >&2
				;;
		esac
	done
}

# ----------------------------------------------------------------------------
# get  result  from the select list.
# ----------------------------------------------------------------------------
get_select()
{
	[ $# -eq 0 ] && return 1

	while prompt "Please enter your selection (1..$# or q) ->"
	do	case ${INPUT:="0"} in
		[0-9]|[0-9][0-9])
			[ $INPUT -lt 1 -o $INPUT -gt $# ] && echo -e "\nError: Invalid option, input a value between 1 and $# \n" && continue
			SEL=$INPUT
			return 0
			;;
		q|Q)	
			SEL="q"
			return 1
			;;
		esac
	done
}

# ----------------------------------------------------------------------------
# Select an item from the list
# ----------------------------------------------------------------------------
select_list()
{
	[ $# -eq 0 ] && return 1

	col=`expr $# / 10 + 1`
	cnt=0
	for option in "$@"
	do	
		cnt=`expr $cnt + 1`
		echo -en "\t$cnt) $option"
		echo $# $col $cnt
		[ `expr $cnt % $col` -eq 0 ] && echo ""
	done
	echo -e "\tq) quit\n"
	get_select "$@"
}


show_status()
{
	if [ "$LOG_WFLAG" == "YES" ]; then
		echo "" >> "$LOG_FILE"
		echo -ne "$(LANG=C date) : \t\t\t\t\t\t[ $1 ]\n" >> "$LOG_FILE"
	fi
	debug_log "\r\t\t\t\t\t\t\t\t[ $1 ]\n" 1 0
}


error_log_compile()
{
	echo 
	tail -n 50 "$LOG_FILE"
	echo "###########################################################################"
	debug_log "$1"
	show_status Failure
	echo "###########################################################################"
	exit 1
}


backup_file()
{
	\cp -a "$1" "$1.$(date +%F-%k-%M)"
	if [ $? -ne 0 ]; then
		return 1
	fi
}


check_file()
{
	if [ ! -f "$1" ]; then
		echo "***************************************************************************"
		error_log "$1  can not found in the machine\n"
		echo "***************************************************************************"
		return 1
	fi
}



check_package()
{
	need_packages=" "
	debug_log "Checking for C development tools ..."
	eval "rpm -q bison > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "gcc --version 2> /dev/null > /dev/null"
		if [ $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"gcc "
		fi
	fi
		
	debug_log "Checking for C++ developement tools ..."
	eval "rpm -q gcc-c++ > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else	
		eval "g++ --version 2> /dev/null > /dev/null"
		if [ $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"gcc-c++ "
		fi
	fi
	
	debug_log "Checking for Make utility ..."
	eval "rpm -q make > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "make --version > /dev/null 2> /dev/null"	
		if [ $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"make "
		fi
	fi
	
	debug_log "Checking for ncurses library ... "
	eval "rpm -q ncurses > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "type clear 2> /dev/null > /dev/null"
		if [  $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"ncurses "
		fi
	fi
	
	debug_log "Checking for ncurses-devel library ... "
	eval "rpm -q ncurses-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [ ! -f "/usr/include/ncurses.h" ] && [ ! -f "/usr/include/ncurses/ncurses.h" ]; then
			show_status	FAILURE
			need_packages=$need_packages"ncurses-devel "
		else
			show_status	OK
		fi
	fi
	
	debug_log "Checking for Perl developement tools ..."
	eval "rpm -q perl > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "perl --version 2>/dev/null > /dev/null"
		if [ $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"perl "
		fi
	fi
	
	debug_log "Checking for Patch ..."
	eval "rpm -q patch > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "patch --version 2> /dev/null > /dev/null"
		if [ $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"patch "
		fi
	fi
	
	debug_log "Checking for bison..."
	eval "rpm -q bison > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "type bison 2> /dev/null > /dev/null"
		if [  $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"bison "
		fi
	fi
	
	debug_log "Checking for bison-devel..."
	eval "rpm -q bison-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [  -f /usr/lib/liby.a ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"bison-devel "
		fi
	fi
	
	debug_log "Checking for openssl..."
	eval "rpm -q openssl > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		eval "type openssl 2> /dev/null > /dev/null"
		if [  $? -eq 0 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"openssl "
		fi
	fi
	
	debug_log "Checking for openssl-devel..."
	eval "rpm -q openssl-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [ -f /usr/include/openssl/ssl.h ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"openssl-devel "
		fi
	fi
	
	debug_log "Checking for gnutls-devel..."
	eval "rpm -q gnutls-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [ -f /usr/include/gnutls/gnutls.h ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"gnutls-devel "
		fi
	fi

	debug_log "Checking for zlib..."
	eval "rpm -q zlib > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [ -f /usr/lib/libz.so.1 ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"zlib "
		fi
	fi

	debug_log "Checking for zlib-devel..."
	eval "rpm -q zlib-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		if [ -f /usr/include/zlib.h ]; then
			show_status	OK
		else
			show_status	FAILURE
			need_packages=$need_packages"zlib-devel "
		fi
	fi

	debug_log "Checking for kernel development packages..."
	eval "rpm -q kernel-devel-$(uname -r) > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		show_status	FAILURE
		need_packages=$need_packages"kernel-devel-$(uname -r) "
	fi
	
	debug_log "Checking for libxml2-devel..."
	eval "rpm -q libxml2-devel > /dev/null"
	if [ $? -eq 0 ]; then
		show_status	OK
	else
		show_status	FAILURE
		need_packages=$need_packages"libxml2-devel "
	fi

	debug_log "Checking for gtk2...\n"
	eval "rpm -q gtk2-devel > /dev/null"
        if [ $? -eq 0 ]; then
                show_status     OK
        else
		yes|yum install gtk2-devel
        fi

	
	echo
	if [  "$need_packages" !=  " " ]; then
		echo "WARNING: Your machine is must prepare some necessary packages"
		debug_log "Necessary Packages $need_packages\n"
		for package in $need_packages
		do
			case $package in
				gcc)
					echo -e "\n C Compiler (gcc)."
					echo -e "    Required for compiling packages."
					echo -e "    Install gcc package (e.g yum install gcc)."
				;;
				g++)
					echo -e "\n C++ Compiler (g++)."
					echo -e "    Required for compiling packages."
					echo -e "    Install gcc-c++ package (e.g yum install gcc-c++)."
				;;
				make)
					echo -e "\n make utility."
					echo -e "    Required for compiling packages."
					echo -e "    Install make package (e.g yum install make)."
				;;
				bash)
					echo -e "\n Bash v2 or greater."
					echo -e "    Required for installation and configuration scripts."
				;;
				ncurses)
					echo -e "\n ncurses library."
					echo -e "    Required for compiling packages."
					echo -e "    Install ncurses development package (e.g yum install ncurses)."
				;;
				ncurses-devel)
					echo -e "\n ncurses-devel library."
					echo -e "    Required for compiling packages."
					echo -e "    Install ncurses development package (e.g yum install ncurses-devel)."
				;;
				perl)
					echo -e "\n Perl development tools."
					echo -e "    Required for compiling packages."
					echo -e "    Install ncurses Perl package (e.g yum install perl)."
				;;
				patch)
					echo -e "\n Patch ."
					echo -e "    Required for compiling packages."
					echo -e "    Install Patch package (e.g yum install patch)."
				;;
				bison)
					echo -e "\n Bison."
					echo -e "    Required for compiling packages."
					echo -e "    Install bison package (e.g yum install bison)."
				;;
				bison-devel)
					echo -e "\n Bison library."
					echo -e "    Required for compiling packages."
					echo -e "    Install bison-devel package (e.g yum install bison-devel)."
				;;
				openssl)
					echo -e "\n OpenSSL."
					echo -e "    Required for compiling packages."
					echo -e "    Install openssl package (e.g yum install openssl)."
				;;
				openssl-devel)
					echo -e "\n OpenSSL library."
					echo -e "    Required for compiling packages."
					echo -e "    Install openssl-devel package (e.g yum install openssl-devel)."
				;;
				gnutls-devel)
					echo -e "\n Gnutls library."
					echo -e "    Required for compiling packages."
					echo -e "    Install gnutls-devel package (e.g yum install gnutls-devel)."
				;;
				zlib)
					echo -e "\n Zlib library."
					echo -e "    Required for compiling packages."
					echo -e "    Install gnutls-devel package (e.g yum install zlib)."
				;;
				zlib-devel)
					echo -e "\n Zlib development packages."
					echo -e "    Required for compiling packages."
					echo -e "    Install zlib-devel package (e.g yum install zlib-devel)."
				;;
				kernel-devel)
					echo -e "\n Kernel development packages."
					echo -e "    Required for compiling packages."
					echo -e "    Install kernel-devel package (e.g yum install kernel-devel)."
				;;
				libxml2-devel)
					echo -e "\n libxml2 development packages."
					echo -e "    Required for compiling packages."
					echo -e "    Install libxml2-devel package (e.g yum install libxml2-devel)."
				;;
#				gtk2-devel)
#					echo -e "\n gtk2-devel development packages."
#					echo -e "    Required for compiling packages."
#					echo -e "    Install gtk2-devel package (e.g yum install gtk2-devel)."
#				;;
			esac
		done

		echo
#		get_res "Do you want  to install the necessary packages"
#		if [ $? -eq 0 ]; then
	        yes|yum install $need_packages
#		fi
#		wait
	fi
	
	return 0
}


get_elastix_version()
{
	SYSTEM_VERSION=`uname -o`	
	# get elastix version
	if [ -f "/root/install.log" ]; then
		NOW_ELASTIX_VERSION=`cat /root/install.log | grep elastix-1 | cut -d' ' -f2`
		if [ "$NOW_ELASTIX_VERSION" = "" ]; then 
		   NOW_ELASTIX_VERSION=`cat /root/install.log | grep elastix-2 | cut -d' ' -f2`
		fi
		NOW_ELASTIX_VERSION=${NOW_ELASTIX_VERSION%.noarch}
                if [ -n "$NOW_ELASTIX_VERSION" ]; then
                        SYSTEM_VERSION="other_elastix_version"
                fi
		
	fi

	debug_log "system version = $SYSTEM_VERSION $NOW_ELASTIX_VERSION\n" 0
	
	return 0
}

set_elastix_install_ver()
{

if [ "$NOW_ELASTIX_VERSION"x = "elastix-1.6-12"x ] ; then
    DAHDI_SRC_RPM="dahdi-2.2.0.2-6.src.rpm"
    ASTERISK_SRC_RPM="asterisk-1.4.26.1-4.src.rpm"
    DAHDI_SRC_DIR="/usr/src/redhat/SOURCES/dahdi-linux-complete-2.2.0.2+2.2.0"
    ASTERISK_SRC_DIR="/usr/src/redhat/SOURCES/asterisk-1.4.26.1"
    ASTERISK_DOWNLOAD_URL="http://repo.elastix.org/elastix/1.6/base/SRPMS/$ASTERISK_SRC_RPM"
    DAHDI_DOWNLOAD_URL="http://repo.elastix.org/elastix/1.6/base/SRPMS/$DAHDI_SRC_RPM"
    AX4G_DAHDI_DIR="$PROD_HOME/src/1.6/dahdi"
    AX4G_ASTERISK_SRC="$PROD_HOME/src/1.6/asterisk"
elif [ "$NOW_ELASTIX_VERSION"x = "elastix-2.0.0-57"x ] ; then
    DAHDI_SRC_RPM="dahdi-2.3.0.1-3.src.rpm"
    ASTERISK_SRC_RPM="asterisk-1.6.2.13-0.src.rpm"
    DAHDI_SRC_DIR="/usr/src/redhat/SOURCES/dahdi-linux-complete-2.3.0.1+2.3.0"
    ASTERISK_SRC_DIR="/usr/src/redhat/SOURCES/asterisk-1.6.2.13"
    ASTERISK_DOWNLOAD_URL="http://repo.elastix.org/elastix/2.0/updates/SRPMS/$ASTERISK_SRC_RPM"
    DAHDI_DOWNLOAD_URL="http://repo.elastix.org/elastix/2.0/updates/SRPMS/$DAHDI_SRC_RPM"
    AX4G_DAHDI_DIR="$PROD_HOME/src/2.0/dahdi"
    AX4G_ASTERISK_SRC="$PROD_HOME/src/2.0/asterisk-1.6.2.13"
    debug_log "Checking for libxml2-devel..."
    eval "rpm -q libxml2-devel > /dev/null"
    if [ $? -eq 0 ]; then
	show_status	OK
    else
        yes|yum -y install libxml2-devel
    fi
else #elastix-2.0.0-32
    DAHDI_SRC_RPM="dahdi-2.3.0.1-3.src.rpm"
    ASTERISK_SRC_RPM="asterisk-1.6.2.10-1.src.rpm"
    DAHDI_SRC_DIR="/usr/src/redhat/SOURCES/dahdi-linux-complete-2.3.0.1+2.3.0"
    ASTERISK_SRC_DIR="/usr/src/redhat/SOURCES/asterisk-1.6.2.10"
    ASTERISK_DOWNLOAD_URL="http://repo.elastix.org/elastix/2.0/updates/SRPMS/$ASTERISK_SRC_RPM"
    DAHDI_DOWNLOAD_URL="http://repo.elastix.org/elastix/2.0/updates/SRPMS/$DAHDI_SRC_RPM"
    AX4G_DAHDI_DIR="$PROD_HOME/src/2.0/dahdi"
    AX4G_ASTERISK_SRC="$PROD_HOME/src/2.0/asterisk"
    debug_log "Checking for libxml2-devel..."
    eval "rpm -q libxml2-devel > /dev/null"
    if [ $? -eq 0 ]; then
	show_status	OK
    else
        yes|yum -y install libxml2-devel
    fi
fi

}


download_ungzip()
{
  echo $ASTERISK_DOWNLOAD_URL
  echo $DAHDI_DOWNLOAD_URL
  
  wget  -t 10 -c $ASTERISK_DOWNLOAD_URL
	
  wget  -t 10 -c $DAHDI_DOWNLOAD_URL

  mkdir -p /usr/src/redhat/SOURCES/
  mkdir -p /usr/src/redhat/SPECS/

  if [ -d $ASTERISK_SRC_DIR ]; then
	\mv "$ASTERISK_SRC_DIR" "$ASTERISK_SRC_DIR.$(date +%F-%k-%M)"
  fi
  
  if [ -d $DAHDI_SRC_DIR ]; then 
	\mv "$DAHDI_SRC_DIR" "$DAHDI_SRC_DIR.$(date +%F-%k-%M)"
  fi

  rpm -ivh $DAHDI_SRC_RPM
  rpm -ivh $ASTERISK_SRC_RPM

  cd /usr/src/redhat/SOURCES/

  tar -zxf $DAHDI_SRC_DIR.tar.gz
  sleep 1
  tar -zxf $ASTERISK_SRC_DIR.tar.gz
  sleep 1

  chmod -R 755 *
  cd $PROD_HOME
}


check_dahdi_src_files()
{
        local flag
        flag=0

        if [ ! -d "$DAHDI_SRC_DIR" ]; then
                error_log "$DAHDI_SRC_DIR is not find in machine\n"
                exit 1
        fi

        cd "$DAHDI_SRC_DIR"

        for file in $*
        do
                check_file "$DAHDI_SRC_DIR/$file"
                if [ $? -ne 0 ]; then
                        flag=1
                fi
        done

        if [ $flag -eq 1 ]; then
		error_log "$DAHDI_SRC_DIR have no modules.sample files\n"
                exit 1
        fi

        echo ""
        echo "***************************************************************************"
        debug_log "Checking dahdi src  files ...           [ OK ]\n"
        echo "***************************************************************************"

        cd "$PROD_HOME"

        sleep 1
}

check_dahdi_patch_files()
{
        local flag
        flag=0

        if [ ! -d "$DAHDI_SRC_DIR" ]; then
                error_log "$DAHDI_SRC_DIR not found\n"
                exit 1
        fi

        if [ ! -d "$PROD_PATCH_DIR/dahdi" ]; then
                debug_log "$PROD_PATCH_DIR/dahdi dir is not found in machine\n"
                exit 1
        fi

        cd "$PROD_PATCH_DIR/dahdi"
        dahdi_patch_files="Kbuild.sed"
        dahdi_patch_files="$dahdi_patch_files ""modules.sample.sed"

        for file in $dahdi_patch_files
        do
                check_file "$PROD_PATCH_DIR/dahdi/$file"
                if [ $? -ne 0 ]; then
                        flag=1
                fi
        done

        if [ $flag -eq 1 ]; then
		debug_log "$PROD_PATCH_DIR/dahdi have no patch file\n"
                exit 1
        fi

        echo
        echo "***************************************************************************"
        debug_log "Checking dahdi Kbuild modules  patch files ...                   [ OK ]\n"
        echo "***************************************************************************"
        cd "$PROD_HOME"

        sleep 1

}


backup_dahdi_tool_module_config()
{
        local flag
        flag=0

        cd "$DAHDI_SRC_DIR"

        for file in $*
        do
                backup_file "$file"
                if [ $? -ne 0 ]; then
                        flag=1
                fi
        done

        if [ $flag -eq 1 ]; then
                exit 1
        fi

        echo
        echo "****************************************************************************"
        debug_log "Backup  dahdi tool dir configures files ...            [ OK ]\n"
        echo "****************************************************************************"

        cd "$PROD_HOME"

        sleep 1
}

patch_dahdi_config()
{
        if [ $DAHDI_SRC_DIR == "" ]; then
                echo "Error : DAHDI SOURCE DIR NOT FOUND !"
                exit 1
        fi

	cd $DAHDI_SRC_DIR
        DAHDI_TOOL_CONFIG_FILES="tools/modules.sample"

	KBUILD_FILE="$DAHDI_SRC_DIR/linux/drivers/dahdi/Kbuild"

        # check dahdi tool dir modules.sample  file
        check_dahdi_src_files $DAHDI_TOOL_CONFIG_FILES
	check_file $KBUILD_FILE

        # check *.sed file
        check_dahdi_patch_files

        # backup dahdi configuration files
	if [ $DAHDI_TOOL_CONFIG_NEED_BACKUP == "YES" ]; then 
        	backup_dahdi_tool_module_config $DAHDI_TOOL_CONFIG_FILES
	fi
	
        #patch /etc/dahdi/modules
        file_modules="$DAHDI_SRC_DIR/tools/modules.sample"
        grep -q "^ax4g" $file_modules
        if [ $? -ne 0 ]; then
                file_modules_sample_sed="$PROD_PATCH_DIR/dahdi/modules.sample.sed"
                sed -i "/^xpp_usb/r $file_modules_sample_sed" $file_modules
        fi

#	grep -q "^ax1600p" $file_modules
#	if [ $? -eq 0 ]; then
#               sed -i "s/^ax1600p/#ax1600p/g" $file_modules
#       fi



	#patch Kbuild add ax4g module

	
        eval "grep -q ax4g $KBUILD_FILE"
        if [ $? -ne 0 ]; then
                backup_file "$KBUILD_FILE"
                Kbuild_sed="$PROD_PATCH_DIR/dahdi/Kbuild.sed"
                sed -i "/CONFIG_DAHDI_TRANSCODE/r $Kbuild_sed" $KBUILD_FILE
        fi

        eval "grep -q ax1600p $KBUILD_FILE"
        if [ $? -ne 0 ]; then
                Kbuild_sed="$PROD_PATCH_DIR/dahdi/Kbuild_ax1600p.sed"
                echo "ready to patch ax1600p in Kbuild"
                sed -i "/CONFIG_DAHDI_TRANSCODE/r $Kbuild_sed" $KBUILD_FILE
        fi

	sleep 1

	cd $PROD_HOME
        return 0
}

cp_ax4g_dahdi_file()
{
      eval "\cp -r $AX4G_DAHDI_DIR/ax4g.c $DAHDI_SRC_DIR/linux/drivers/dahdi/"
      eval "\cp -r $AX4G_DAHDI_DIR/dahdi-base.c $DAHDI_SRC_DIR/linux/drivers/dahdi/"
      eval "\cp -r $AX4G_DAHDI_DIR/user.h $DAHDI_SRC_DIR/linux/include/dahdi/"
      eval "\cp -r $AX4G_DAHDI_DIR/kernel.h $DAHDI_SRC_DIR/linux/include/dahdi/"
      eval "\cp -r $AX4G_DAHDI_DIR/dahdi_debug.h $DAHDI_SRC_DIR/linux/drivers/dahdi/xpp/"
      eval "\cp -r $AX4G_DAHDI_DIR/ax1600p.c $DAHDI_SRC_DIR/linux/drivers/dahdi/"
}

cp_ax4g_asterisk_file()
{
      \cp -f $AX4G_ASTERISK_SRC/res_clisendsms.c $ASTERISK_SRC_DIR/res/
      \cp -f $AX4G_ASTERISK_SRC/app_sendsms.c $ASTERISK_SRC_DIR/apps/
        \cp -f $AX4G_ASTERISK_SRC/pbx.c $ASTERISK_SRC_DIR/main/
        \cp -f $AX4G_ASTERISK_SRC/asterisk.c $ASTERISK_SRC_DIR/main/
       \cp -f $AX4G_ASTERISK_SRC/cli.c $ASTERISK_SRC_DIR/main/
        \cp -f $AX4G_ASTERISK_SRC/pbx.h $ASTERISK_SRC_DIR/include/asterisk/
        \cp -f $AX4G_ASTERISK_SRC/chan_dahdi.c $ASTERISK_SRC_DIR/channels/
}

patch_elastix()
{
    patch_dir="/usr/src/redhat/SOURCES"

    if [ "$NOW_ELASTIX_VERSION"x = "elastix-1.6-12"x ] ; then
      \cp -f $patch_dir/dahdi-ax1600p-support.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-ax1600p-tools-support.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.1.0.4_zaphfc.diff $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.2.0.2-sync-to-rsp-001-odistuff.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.2.0.2-sync-to-rsp-003-dahdi_timing.diff $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-openvox-elastix.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi_2.1.0.4_wcte11xp.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-bri_dchan-2.2.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-list-bri-cards.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-load-extra-modules.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-recognize-ap400.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-recognize-sangoma-t1.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-oslec2.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/redfone_ethmf_driver_for_dahdi.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/ystdm-8-16-dahdi.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.2.0.1-backport-r6785.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.2.0.2-bug15647-svn6933-do_not_lockup_with_dahdi_dummy.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-ap400-support.patch $DAHDI_SRC_DIR
   else
      \cp -f $patch_dir/dahdi-oslec2.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-ap400-support.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi_2.1.0.4_wcte11xp.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-openvox-elastix.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/ystdm-8-16-dahdi.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-recognize-ap400.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-recognize-sangoma-bri.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-ax1600p-support.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-ax1600p-tools-support.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-bri_dchan-2.2.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-sync-to-rsp-001-odistuff.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-load-extra-modules.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0.1-xpp_fxloader-svn8580-svn8589-delayed-load.diff $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0.1-zaphfc.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-also-unload-oslec.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-ap400-rename-errcount.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0-list-bri-cards.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.2.0.2-wcopenpci.patch $DAHDI_SRC_DIR
      \cp -f $patch_dir/dahdi-2.3.0.1-add-span-owner.patch $DAHDI_SRC_DIR
   fi
   # cp patch elastix script to run 
      \cp $PROD_PATCH_DIR/patch_dahdi.sh $DAHDI_SRC_DIR
      eval "\cp -r $AX4G_DAHDI_DIR/*.patch $DAHDI_SRC_DIR/"
      cd $DAHDI_SRC_DIR
      chmod +x  ./patch_dahdi.sh
      ./patch_dahdi.sh $NOW_ELASTIX_VERSION  
       cp_ax4g_dahdi_file
       cd $PROD_HOME
}

try_check_dahdi_modules_config()
{
        grep -q "^ax4g" $DAHDI_MODULES_CFG
        if [ $? -ne 0 ]; then
                backup_file "$DAHDI_MODULES_CFG"
                file_modules_sample_sed="$PROD_PATCH_DIR/dahdi/modules.sample.sed"
                sed -i "/wctdm24xxp/r $file_modules_sample_sed" $DAHDI_MODULES_CFG
        fi
	
	#grep -q "^ax1600p" $DAHDI_MODULES_CFG
        #if [ $? -eq 0 ]; then
        #        sed -i "s/^ax1600p/#ax1600p/g" $DAHDI_MODULES_CFG
        #fi

        echo
        echo "***************************************************************************"
        debug_log "Installing Atcom ax4g driver ...                   [ OK ]\n"
        echo "***************************************************************************"

        sleep 1
}

install_dahdi()
{
	echo 
	echo "********************************************"
        echo "ready to make dahdi"
	echo "********************************************"
	echo 
	
	mkdir -p  $PROD_HOME/bak
	\mv  "$DAHDI_MODULES_DIR"  "$PROD_HOME/bak/dahdi.$(date +%F-%k-%M)"
	
        cd $DAHDI_SRC_DIR/
        echo $LOG_FILE
        make >> "$LOG_FILE"
        if [ $? -ne 0 ]; then
                echo
                echo -e "ERROR: Installing $DAHDI_SRC_DIR: make ...                   [ FAILURE ]\n"
                exit 1
        fi
        make install >> "$LOG_FILE"
        if [ $? -ne 0 ]; then
                echo
                echo -e "ERROR: Installing $DAHDI_SRC_DIR: make install ...           [ FAILURE ]\n"
                exit 1
        fi
	
	if [ ! -f $DAHDI_MODULES_CFG ]; then
		make config >> "$LOG_FILE"
		if [ $? -ne 0 ]; then
                echo
                echo -e "ERROR: Installin $DAHDI_SRC_DIR : make config ...           [ FAILURE ]\n"
                exit 1
		else
		echo "make config dahdi....        [OK]"
        	fi
	else
		echo "system is already make config dahdi"

	fi
	   
        echo
        echo "***************************************************************************"
        debug_log "make && make install  dahdi driver  ...                   [ OK ]\n"
        echo "***************************************************************************"

#	/etc/init.d/dahdi restart
#	
#	if [ ! -f /proc/dahdi/ax4g_sta_card_1 ]; then
#		/etc/init.d/dahdi restart
#	fi
#		
#	if [ -f /etc/asterisk/chan_dahdi.conf ]; then
#		dahdi_genconf >>"$LOG_FILE"
#
#       	if [ $? -ne 0 ]; then
#             	echo
#               	echo -e "ERROR: Installing $DAHDI_CFG_DIR configure...          [ FAILURE ]\n"
#		else
#			echo -e "Installing dahdi channel configures [OK]\n"
#        	fi
#
#		grep -q "^#include dahdi-channels.conf" "$AST_CONFIG_DIR/chan_dahdi.conf" 
#       	if [ $? -ne 0 ]; then
#               	echo  "#include dahdi-channels.conf" >> $AST_CONFIG_DIR/chan_dahdi.conf
#       	fi
#
#	fi
	
        cd $PROD_HOME
        sleep 1
}

patch_asterisk()
{
    patch_dir="/usr/src/redhat/SOURCES"

    if [ "$NOW_ELASTIX_VERSION"x = "elastix-1.6-12"x ] ; then
      \cp -f  $patch_dir/asterisk-1.4.0-beta3-user.patch $ASTERISK_SRC_DIR
      \cp -f  $patch_dir/asterisk-1.0.7-mpg123.patch $ASTERISK_SRC_DIR
      \cp -f  $patch_dir/asterisk-1.4-spandsp.patch $ASTERISK_SRC_DIR
      \cp -f  $patch_dir/asterisk-rc-chown.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/13892_2.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/openr2-asterisk-1.4.24.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-h323-use-pboolean-only.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-g722.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-rc-kill-safe_asterisk-first.patch  $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4.25.1_chan_dahdi_bris.diff $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-select-moh-opsound.patch  $ASTERISK_SRC_DIR
    elif [ "$NOW_ELASTIX_VERSION"x = "elastix-2.0.0-57"x ] ; then
      \cp -f $patch_dir/asterisk-1.4.0-beta3-user.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-rc6-mpg123.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4-spandsp.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-rc-chown.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/13892_2.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/openr2-asterisk-1.4.24.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-h323-use-pboolean-only.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-g722.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4.25.1_chan_dahdi_bris.diff $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-rc3-app_fax_compat_name.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-unselect-moh-wav.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-select-moh-gsm.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-cdr_sqlite3_custom.conf.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.13-rename-moh-mohmp3.patch $ASTERISK_SRC_DIR
    else
      \cp -f $patch_dir/asterisk-rc-chown.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/13892_2.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-unselect-moh-wav.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-select-moh-gsm.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-cdr_sqlite3_custom.conf.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/openr2-asterisk-1.4.24.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-h323-use-pboolean-only.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-g722.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4.0-beta3-user.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-rc6-mpg123.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4-spandsp.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.4.25.1_chan_dahdi_bris.diff $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.0-rc3-app_fax_compat_name.patch $ASTERISK_SRC_DIR
      \cp -f $patch_dir/asterisk-1.6.2.6-bug14577-try2.patch $ASTERISK_SRC_DIR
   fi
      # cp patch_asterisk.sh and run 

      \cp $PROD_PATCH_DIR/patch_asterisk.sh $ASTERISK_SRC_DIR
       \cp -f $AX4G_ASTERISK_SRC/*.patch  $ASTERISK_SRC_DIR/
       cd $ASTERISK_SRC_DIR
      
      chmod +x ./patch_asterisk.sh
      ./patch_asterisk.sh $NOW_ELASTIX_VERSION

      cp_ax4g_asterisk_file
      echo "***************************************************************************"
      debug_log "Patching asterisk  files for $NOW_ELASTIX_VERSION ...         [ OK ]\n"
      echo "***************************************************************************"

      cd $PROD_HOME

      sleep 1
}

install_asterisk()
{
        #cp extra modules file
	
        cd $ASTERISK_SRC_DIR
	echo "Installing asterisk must to stop asterisk running"
	
	if [ ! "$NOW_ELASTIX_VERSION" ]; then
	  	killall -9 asterisk
	else
		amportal stop
    	fi

	#stop asterisk
        echo  "make  asterisk, it will last a few minutes, wait for a moment"
        ./configure >> "$LOG_FILE"
        if [ $? -ne 0 ]; then
                echo
                echo  "ERROR: Installing Asterisk: ./configure ...                   [ FAILURE ]"
                exit 1
        fi

        make >> "$LOG_FILE"
        if [ $? -ne 0 ]; then
                echo
                echo "ERROR: Installing $ASTERISK_SRC_DIR: make ...                 [ FAILURE ]"
                exit 1
        fi
	
	backup_file "$AST_SBIN_DIR/asterisk"
	backup_file "$AST_INCLUDE_DIR/pbx.h" 

	if [ ! -d $AST_MODULES ]; then
	 	make install >> "LOG_FILE"
	else
	\cp -a  $AST_MODULES/chan_dahdi.so $AST_MODULES/chan_dahdi.so.$(date +%F-%k-%M)
        \cp -rf $ASTERISK_SRC_DIR/channels/chan_dahdi.so $AST_MODULES
        \cp -rf $ASTERISK_SRC_DIR/res/res_clisendsms.so  $AST_MODULES
        \cp -rf $ASTERISK_SRC_DIR/apps/app_sendsms.so $AST_MODULES
        \cp -rf $ASTERISK_SRC_DIR/include/asterisk/pbx.h $AST_INCLUDE_DIR
        \cp -rf $ASTERISK_SRC_DIR/main/asterisk $AST_SBIN_DIR
	fi

	if [ ! -d $AST_CONFIG_DIR ]; then
		make samples >> "$LOG_FILE"
	fi

#	if [ -f /usr/sbin/dahdi_genconf ]; then
#		echo "execute dahdi_genconf for dahdi"
#		/etc/init.d/dahdi restart 
#		dahdi_genconf
#		grep -q "^#include dahdi-channels.conf" "$AST_CONFIG_DIR/chan_dahdi.conf"
#               if [ $? -ne 0 ]; then   
#                echo "#include dahdi-channels.conf" >> $AST_CONFIG_DIR/chan_dahdi.conf
#               fi
#
#	
#	fi 

        echo
        echo "***************************************************************************"
        debug_log "make && make install  Asterisk ...                   [ OK ]\n"
        echo "***************************************************************************"

        cd $PROD_HOME
        sleep 1
}

install_sendmsg()
{
	
        echo "***************************************************************************"
	debug_log "make cli send msg for asterisk\n"
        echo "***************************************************************************"
	cd $PROD_HOME/src
	gcc -O2 -o send send.c 

	if [ ! -d /var/lib/asterisk/agi-bin ]; then
		error_log "/var/lib/asterisk/agi-bin dir is not find in machine\n"
	fi
		
	\cp send /var/lib/asterisk/agi-bin
	\cp send /usr/bin
}

restart_dahdi_asterisk()
{
	/etc/init.d/dahdi restart >> "$LOG_FILE"
	
	amportal restart 

        if [ ! -f /proc/dahdi/ax4g_sta_card_1 ]; then
		echo "ax4g_sta_card_1 is not find and restart dahdi again"
                /etc/init.d/dahdi restart 
        fi

        if [ -f /etc/asterisk/chan_dahdi.conf ]; then

		echo "ax4g want to genconf in /etc/asterisk/chan_dahdi.conf"
		
		backup_file "$AST_CONFIG_DIR/chan_dahdi.conf"

                dahdi_genconf >>"$LOG_FILE"

                if [ $? -ne 0 ]; then
                        echo
                        echo -e "ERROR: Installing $DAHDI_CFG_DIR configure...          [ FAILURE ]\n"
                else
                        echo -e "Installing dahdi channel configures [OK]\n"
                fi

                grep -q "^#include dahdi-channels.conf" "$AST_CONFIG_DIR/chan_dahdi.conf"
                if [ $? -ne 0 ]; then
			dahdi_genconf 
                fi
	else
		debug_log "ax4g want to start dahdi but chan_dahdi.conf is not find\n"

        fi
	
	/etc/init.d/dahdi restart >> "$LOG_FILE"
	asterisk -rx "dahdi restart"
}

install_ax4g()
{
		
	if [ ! "$SYSTEM_VERSION" ]; then
		error_log "the matchine System is invalid to the script\n"
		exit 1
	fi

	if [ ! "$NOW_ELASTIX_VERSION" ]; then
#		error_log " elastix version is invalid to the script\n"
#		exit 1
		
		if [ "$SUPPORT_BARE_INSTALL" != "bare" ]; then
			exit 1
		else               
			debug_log "you have not install elastix now , so will install asterisk-1.6.2.10  dahdi-2.3.0.1 for you\n"
                	get_res "Do you want to install:[y/n]"
			if [ $? -eq 0 ]; then
                  		:
			else
		  		exit 1
                	fi
		fi

	fi
	

	if [ "$NOW_ELASTIX_VERSION" == "elastix-1.6-12" ]; then
		debug_log "Elastix Version = $NOW_ELASTIX_VERSION\n"
	elif [ "$NOW_ELASTIX_VERSION" == "elastix-2.0.0-32" ]; then
	  debug_log "Elastix Version = $NOW_ELASTIX_VERSION\n"
	elif [ "$NOW_ELASTIX_VERSION" == "elastix-2.0.0-57" ]; then
	  debug_log "Elastix Version = $NOW_ELASTIX_VERSION\n"
	elif [ "$NOW_ELASTIX_VERSION" != "" ]; then 
		error_log "The current version was not support the $NOW_ELASTIX_VERSION\n"
		exit 1
	else
		NOW_ELASTIX_VERSION = "elastix-2.0.0-3";
	fi

	set_elastix_install_ver	

	download_ungzip

	if [ ! -d $AST_CONFIG_DIR ]; then
		patch_asterisk

        	install_asterisk

        	install_sendmsg

		patch_elastix

                patch_dahdi_config

                install_dahdi

                try_check_dahdi_modules_config

	else
		patch_elastix

		patch_dahdi_config
	
		install_dahdi

        	try_check_dahdi_modules_config

		patch_asterisk

        	install_asterisk

		install_sendmsg
	fi

#	rm -rf $ASTERISK_SRC_DIR
#	rm -rf $DAHDI_SRC_DIR
	
	restart_dahdi_asterisk
	
	display
	echo -e "\n\n"

	echo ""
	echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
	debug_log "Finish install  Atcom  ax4g card driver for $NOW_ELASTIX_VERSION !\n"
	echo -e "  You can use the list commands to show ax4g card status :          \n"
	echo "		asterisk -rx "dahdi show channels"		               "
	echo " 		dahdi_cfg -vvv                                                 "            
	echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
	echo -e "\n" 
}


# set log file empty
cat /dev/null > "$LOG_FILE"

# check permission 
if [ "$UID" -ne "$EXECUTE_UID" ]
then
	echo "you do not have permission to execute"
	exit 1
fi 

# check bash version
check_sh

# check necessary package
#check_package



# show title of running
display

# get elastix  version
get_elastix_version

install_ax4g

